#include <winsock2.h>
#include <iostream.h>
#include "PacketMuncher.h"
#include "Packets.h"
#include "conmap.h"


unsigned long PacketMuncher::localHost;

PacketMuncher::PacketMuncher(){
	sock = socket(AF_INET, SOCK_RAW,IPPROTO_IP);
	if(sock==INVALID_SOCKET ){
		cout << "Error creating a socket: "<< err() << endl;
		return;
	}

	sockaddr_in addr;
	memset(&addr, 0, sizeof(addr));
	localHost = getAddr();
	addr.sin_addr.s_addr = localHost; // bind to a local interface (INADDR_ANY does not work)
	addr.sin_family = AF_INET;
	addr.sin_port = htons(6000);
	if( bind(sock,(SOCKADDR *)&addr,sizeof(addr)) == SOCKET_ERROR ){
		cout << "While binding socket: "<< err() << endl;
		return;
	}

	DWORD inBuffer = RCVALL_ON, outBytesReturned;
	if(WSAIoctl(sock, SIO_RCVALL, &inBuffer, sizeof(inBuffer), NULL, 0, &outBytesReturned, NULL , NULL) == SOCKET_ERROR){
		cout << "Error in WSAIoctl: " << err() << endl;
		return;
	}
}


PacketMuncher::~PacketMuncher(){
	closesocket(sock);
}


unsigned long PacketMuncher::getAddr(){
	HOSTENT *hp;
	char hostname[128];
	DWORD dwSize = 128;
	
	if (GetComputerName(hostname, &dwSize) == 0){
	    cout << "ERROR getting name: " << err() << endl;
		return 0;
	}
	if((hp = gethostbyname(hostname)) == NULL){
	    cout << "ERROR getting host by name: " << err() << endl;
		return 0;  
	}
	cout << "name: " << hp->h_name << " addr: "<<inet_ntoa(*(in_addr*)(hp->h_addr)) << endl << endl;
	return * ((unsigned long*)hp->h_addr_list[0]); // h_addr is a macro for h_addr_list[0]
}


int PacketMuncher::munch(SyncLinkedList *list){
	while( recv(sock,buff,sizeof(buff),0) != SOCKET_ERROR){
		bool addBuffer = false;
		IPH* ip = (IPH*)buff;
		unsigned short iphlen = (ip->verlen&0xF)<<2;
		char * transport = buff + iphlen; // + ipHeaderLengthField*4
		TCPH* tcp = (TCPH*) transport;
		ICMPH* icmp = (ICMPH*) transport;
		UDPH* udp = (UDPH*) transport;
//		cout << '|';
		switch(ip->protocol){
			case IPPROTO_TCP:
				if((tcp->Flags&TH_SYN) && !(tcp->Flags&TH_ACK)){ // tcp->Flags^TH_SYN
					addBuffer=true;
				}
				break;
			case IPPROTO_ICMP:
				switch(icmp->type){
					case ICMP_ECHOREP:
					case ICMP_UNREACH:
					case ICMP_TIMEEX:
					addBuffer=true;
				}
				break;
			case IPPROTO_UDP:
/// make sure it's not a broadcast packet
				if( ntohs(udp->srcPort)!=53 && ntohs(udp->destPort)!=53 ){ // if not DNS
					addBuffer=true;
				} 
				break;
		}
		if(addBuffer){
			char * packet = new char[MUNCH_BUFFER_SIZE];
			if(!packet){
				cout << "Could not allocate memory for a network buffer" << endl;
				return 1;
			}
			memcpy(packet,buff,MUNCH_BUFFER_SIZE);
			list->add(packet);
		}
	}
	cout << err();
	return SOCKET_ERROR;
}
