#include <windows.h>
#include <iostream.h>
#include "SyncLinkedList.h"
#include "conmap.h"

SyncLinkedList::SyncLinkedList(){
	head = NULL;
	semaphore = CreateSemaphore(NULL,0,MAX_BUFFERS_IN_QUEUE,"List Semaphore");
	if(!semaphore){
		cout << "Error creating semaphore " << err() << endl;
		return;
	}
	__try{
		InitializeCriticalSection(&critsec);
	}__except( GetExceptionCode() == STATUS_NO_MEMORY ){
		cout << "Out of memory when initializing critical section"<<endl;
	}
}

SyncLinkedList::~SyncLinkedList(){
char * buff;
	while( (buff=get()) ){
		delete(buff);		
	}
}


void SyncLinkedList::add(char *buff){
long previousCount; // semaphore previous count - can be null in ReleaseSemaphore()
ListNode * node;
	EnterCriticalSection(&critsec); /********************************************/
	if( ReleaseSemaphore(semaphore,1,&previousCount) ){ // fails if count > MAX_BUFFERS_IN_QUEUE
		node = new ListNode();
		if(!node){
			cout << endl << "Could not allocate memory for a list node" << endl;
			cout.flush();
		} else { // fix it so it adds it to the end !!!
			node->buff = buff;
			node->next = head;
			head = node;
		}
	} // if semaphore...
	LeaveCriticalSection(&critsec); /*******************************************/
//	cout << "Packets in queue: " << (previousCount+1) << " new node=" << ((unsigned int)node) << endl;
}


char * SyncLinkedList::get(){
	if(WAIT_TIMEOUT == WaitForSingleObject(semaphore,LIST_GET_TIMEOUT)){ // wait for a buffer
		return NULL; // no buffers within specified timeout
	}
	EnterCriticalSection(&critsec); /********************************************/
	ListNode * node = head;
	head = head->next;
	LeaveCriticalSection(&critsec); /*******************************************/
	char * buff = node->buff;
	cout << "removing node=" << (unsigned int)node << endl;
	delete(node);
	return buff;
}
