#include <iostream.h>
#include "Tracker.h"


bool Tracker::isBeingTraced(unsigned int addr){
	for(TrackerNode * node = head; node; node=node->next){
		if(node->tr.getTraceAddress() == addr){
			return true;
		}
	}
	return false;
}


void Tracker::startTrace(unsigned int addr){
	if(!isBeingTraced(addr)){
		cout << "Tracing route to " << inet_ntoa(*(in_addr*)&addr) << endl;
		TrackerNode * node = new TrackerNode();
		if(!node){
			cout << "Out of memory..." << endl;
			return;
		}
		node->tr.initTrace(addr, traceID);
		node->next = head;
		head = node;
		traceID++;
	}
}


void Tracker::ping(MapMaker * map){
	for(TrackerNode * node = head; node;){
		unsigned int * routeTable = node->tr.ping();
		if(routeTable){
			map->addRoute(routeTable);
			node->tr.printRoute();
			node = remove(node);
		} else {
			node=node->next;
		}
	}
}


void Tracker::pong(char * packet){
	for(TrackerNode * node = head; node; node=node->next){
		node->tr.pong(packet);
	}
}


Tracker::Tracker(){
	head = NULL;
	traceID = 0;
}


Tracker::~Tracker(){
	while(head){
		TrackerNode * next = head->next;
		delete(head);
		head = next;
	}
}

// remove node from the list and return the following node so we can continue iterating
TrackerNode * Tracker::remove(TrackerNode *node){
TrackerNode * next;
	if(!node){
		return NULL;
	}
	if(node==head){
		head = head->next;
	} else {
		for(next = head; next; next=next->next){ // if node is in the middle of the list
			if(next->next == node){
				next->next = next->next->next;
				break;
			}
		}
	}
	next = node->next;
	delete(node);
	return next;
}
