#include <iostream.h>
#include "tracert.h"
#include "conmap.h"

int TraceRoute::setTtl (int timeToLive) { 
	if(sock == INVALID_SOCKET){
		cout << "ERROR in TraceRoute::setTtl(): invalid socket" << endl;
		return INVALID_SOCKET;
	}
	int nRet = setsockopt (sock, IPPROTO_IP, IP_TTL, (LPSTR)&timeToLive, sizeof(int)); 
	if (nRet==SOCKET_ERROR) {
		cout << "ERROR setsockopt(lewel=IPPROTO_IP, option=IP_TTL, ttl="<< timeToLive << ") " << err() << endl;
	} 
	return nRet; 
}


// I ripped this one off :)
unsigned short TraceRoute::cksum (unsigned short *buff, int size) { 
	long sum = 0;
	for(sum=0; size > 0; size -=2) { 
		sum += *(buff++);
	}
	sum = (sum & 0xffff) + (sum>>16); 
	sum += (sum >> 16);
	return (unsigned short)(~sum); 
}

TraceRoute::TraceRoute(){
	sock = INVALID_SOCKET;
}

TraceRoute::~TraceRoute(){
	closesocket(sock);
}

int TraceRoute::ping(int ttl) {
	if(sock == INVALID_SOCKET){
		cout << "ERROR in TraceRoute::ping(): ivalid socket" <<endl;
		return INVALID_SOCKET;
	}
	setTtl(ttl);
	packet.type		= ICMP_ECHOREQ;
	packet.code		= 0;
	packet.id		= traceID; // no need to htons
	packet.sequence	= htons( ttl );
	packet.checksum	= 0;
	packet.checksum = cksum((unsigned short *)&packet, sizeof(packet)); 
	int ret = sendto (sock, (char*)&packet, sizeof(packet), 0, (SOCKADDR*)&to, sizeof(to));
	cout << "PING out traceid=" << traceID << " ttl=" << ttl << endl;
	if (ret == SOCKET_ERROR) { 
		cout << "Error in sendto: "<< err() << endl;
	} 
	return ret; 
}

/**********************************************************************************************/

int TraceRoute::initTrace(unsigned int addr, unsigned short id){
	if(sock != INVALID_SOCKET){
		cout << "Already Tracing..." << endl;
		return 0; 
	}
	traceID = id;
	memset(&to, 0, sizeof(to));
	to.sin_family = AF_INET;
	to.sin_addr.s_addr = addr;
	to.sin_port = 0; // ports are not used for icmp

	sock = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
	if(sock==INVALID_SOCKET ){
		cout << "While creating a socket: "<< err() << endl;
		return INVALID_SOCKET;
	}

	for(int i =0; i <= MAX_HOPS; i++){
		route[i]=0;
	}
	route[0]=addr; // save the destination address
	routeLength=0;
	lastTTL = 0;
	scanCount=0;
	ping();
	return 0;
}


unsigned int * TraceRoute::ping(){
	if(sock == INVALID_SOCKET){
		return NULL;
	}
	if(scanCount > 3){
		end();
		return route;
	}
	lastTTL++;
	if(scanCount!=0){ // see if any of the entries were not filled yet
		while(route[lastTTL] && lastTTL < routeLength){
			lastTTL++;
		}
	}
	if((lastTTL > MAX_HOPS) || (routeLength&&(routeLength<=lastTTL)) ){
		scanCount++;
		lastTTL=0;
		return NULL; // we'll continue scanning
	}
	ping(lastTTL);
	return NULL;
}


// pong should be called when type 0,3,11 of icmp packet is captured
int TraceRoute::pong(char *inpacket){
	if(sock == INVALID_SOCKET){ return 1; }
	IPH * ip = (IPH* ) inpacket;
	int iphlen = (ip->verlen&0xF)<<2;
	ICMPH * icmp = (ICMPH*) (inpacket+iphlen);
	IPH * encIP = (IPH*)(inpacket+iphlen+8); // icmp has 4byte header + 4 reserved bytes
	ICMPH * encIcmp = (ICMPH*) (inpacket+iphlen+8+((encIP->verlen&0xF)<<2)); // skip tcp,icmp,tcp to new icmp

	if(icmp->type == ICMP_TIMEEX && icmp->code == 0){ // destination time exceeded from a router
		int seq = ntohs(encIcmp->sequence);
		// if destination is what we are pinging. In our packets id==sequence
		if( (to.sin_addr.s_addr == encIP->daddr) && (encIcmp->id == traceID) ){
			route[seq] = ip->saddr; // record the route
		}
	} else if(icmp->type == ICMP_ECHOREP && ip->saddr == to.sin_addr.s_addr ){ // we have reached the host (echo reply)
		int seq = ntohs(icmp->sequence);
		routeLength = ((!routeLength)||(seq<routeLength)) ? seq : routeLength;
		route[seq] = ip->saddr; // record the last hop
	} else if(icmp->type == ICMP_UNREACH && encIP->daddr == to.sin_addr.s_addr ){
		cout << endl << "Destination unreachable..." << endl;
		end();
	}
	return 0;
}

void TraceRoute::printRoute(){
	if(!routeLength){ // host did not return echo request
		for(int j=1; j <= MAX_HOPS; j++){
			routeLength = route[j] ? j : routeLength;
		}
	}
	cout << endl << "Route to " << inet_ntoa(*(in_addr*)&route[0]) << " (" << routeLength <<"hops):" << endl;
	for(int i=1; i < routeLength; i++){ // route[0] has destination host
		if(route[i]){
			HOSTENT * he = gethostbyaddr((char*)&route[i],4,AF_INET);
			cout << inet_ntoa(*(in_addr*) &route[i] ) <<" "<< (he?he->h_name:"") << endl;
//			cout << inet_ntoa(*(in_addr*) &route[i] ) << endl;
		} else {
			cout << "*" << endl;
		}
	}
	cout << endl;
	cout.flush();
}

int TraceRoute::end(){
	if(sock == INVALID_SOCKET){
		return INVALID_SOCKET;
	}
	cout << "####################### closing socket " << sock << " #####################" <<endl;
	closesocket(sock);
	sock = INVALID_SOCKET;
	return 0;
}

